package com.capitalware.ufmsm.models;

import java.util.ArrayList;
import java.util.Arrays;

import javax.swing.table.AbstractTableModel;

import com.capitalware.ufmsm.objects.CWConstants;

/**
 * This class is the model ("M") of the MVC.
 * It will handle the setting and getting of the data to and from the table.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class StatusQueueModel extends AbstractTableModel
{
   private static final String columnNames[] = new String[]{"Date / Time",
                                                            "Hostname",
                                                            "IP Address",
                                                            "OS",
                                                            "UserID",
                                                            "Exit Code",
                                                            "UFM Version",
                                                            "UFM Workflow",
                                                            };
   private ArrayList tableData;

   /**
    * The constructor
    * @param al
    */
   public StatusQueueModel(ArrayList al)
   {
      super();
      tableData = al;
   }

   /**
    * Retrieve the number of rows in the table.
    * @return rowCount
    */
   public int getRowCount()
   {
      return tableData.size();
   }

   /**
    * Retrieve the number of columns in the table.
    * @return columnCount
    */
   public int getColumnCount()
   {
      return columnNames.length;
   }

   /**
    * Retrieve a value of a particular cell.
    * @param i
    * @param j
    * @return object
    */
   public Object getValueAt(int i, int j)
   {
      if (i > tableData.size())
         return null;
      if (j > columnNames.length)
         return null;

      Object[] rowCells =  (Object[])tableData.get(i);

      return  rowCells[j];
   }

   /**
    * Retrieve the data for an entire row.
    * @param r
    * @return objectArray
    */
   public Object[] getRowData(int r)
   {
      if (r < tableData.size())
         return ((Object[]) tableData.get(r));
      else
         return null;
   }

   /**
    * Retrieve the name of a column..
    * @param i
    * @return columnName
    */
   public String getColumnName(int i)
   {
      return columnNames[i];
   }

   /**
    * Replace the entire table data
    * @param al
    */
   public void setTableData(ArrayList al)
   {
      ArrayList old_al = this.tableData;
      this.tableData = al;
      /*
       * Help GC notice that the old objects are no longer used.
       */
      old_al.clear();
      old_al = null;
   }

   /**
    * Add more data to the table
    * @param al
    */
   public void addTableData(ArrayList al)
   {
      tableData.addAll(al);
   }

   /**
    * Add more data to the table
    * @param newData
    */
   public void addUniqueDataToTable(Object[] newData)
   {
      byte[] newCorrelID = (byte[])newData[CWConstants.statusQcolumns - 1];
      Object[] oo;
      byte[] correlID;
      boolean addToAL = true;
      
      for (int j=0; j < tableData.size(); j++)
      {
         oo = ((Object[]) tableData.get(j));;
         correlID = (byte[])oo[CWConstants.statusQcolumns - 1];
         
         if (Arrays.equals(newCorrelID,correlID))
         {
            addToAL = false;
            break;
         }
      }
      if (addToAL)
         tableData.add(newData);
   }
}
