package com.capitalware.ufmsm.mq;

import java.io.IOException;
import java.net.MalformedURLException;

import javax.swing.*;

import java.security.InvalidAlgorithmParameterException;
import java.security.NoSuchAlgorithmException;
import java.util.ArrayList;
import java.util.Date;
import java.util.StringTokenizer;

import com.capitalware.ufmsm.UFMSM;
import com.capitalware.ufmsm.mq.objects.MQObject;
import com.capitalware.ufmsm.objects.CWConstants;

import com.ibm.mq.*;

/**
 * This class will handle all WMQ interactions.
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class MQGetStatusMessages extends MyMQ
{
   private UFMSM           parent;
   private boolean         working = true;
   private long            startTime;
   private long            endTime;
   private ArrayList       cells = null;

   /**
    * The constructor
    *
    * @param mqo MQObject
    * @param parent (UFM_SM_GUI)
    */
   public MQGetStatusMessages(MQObject mqo, UFMSM parent)
   {
      super(mqo);
      this.parent = parent;
   }

   /**
    * The run method is used to retrieve the MQ channel status
    * information in a separate thread.
    */
   public void run()
   {
      boolean majorFailure = false;
      String  popupTitle = "";
      String  popupMessage = "";
      MQQueue inQ = null;
      int openInputOptions  = MQC.MQOO_INQUIRE + MQC.MQOO_BROWSE + MQC.MQOO_FAIL_IF_QUIESCING + MQC.MQOO_INPUT_SHARED;

      try
      {
         connect();
         
         inQ = _qMgr.accessQueue( mqo.qObject.qName,
                                  openInputOptions,
                                  null,
                                  null,
                                  null );
         
         masterLoop(inQ);
      }
      catch (MQException e)
      {
         popupTitle = "Error";
         popupMessage = "QMgrName = "+mqo.qMgrName+"\nQueue Name = "+mqo.qObject.qName+"\nCompletion Code = " + e.completionCode + "\nReason Code = " + e.reasonCode;
         majorFailure = true;
      }
      catch (MalformedURLException e)
      {
         popupTitle = "Error: MalformedURLException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (InvalidAlgorithmParameterException e)
      {
         popupTitle = "Error: InvalidAlgorithmParameterException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (NoSuchAlgorithmException e)
      {
         popupTitle = "Error: NoSuchAlgorithmException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (ClassNotFoundException e)
      {
         popupTitle = "Error: ClassNotFoundException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (InstantiationException e)
      {
         popupTitle = "Error: InstantiationException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      catch (IllegalAccessException e)
      {
         popupTitle = "Error: IllegalAccessException";
         popupMessage = e.getLocalizedMessage();
         majorFailure = true;
      }
      finally
      {
         if (majorFailure)
         {
            JOptionPane.showMessageDialog( parent,
                                           popupMessage,
                                           popupTitle,
                                           JOptionPane.ERROR_MESSAGE);
         }

         try
         {
            if (inQ != null)
               inQ.close();
         }
         catch (MQException e)
         {
            JOptionPane.showMessageDialog( parent,
                                           "QMgrName = "+mqo.qMgrName+"\nQueue Name = "+mqo.qObject.qName+"\nCompletion Code = " + e.completionCode + "\nReason Code = " + e.reasonCode,
                                           "Error",
                                           JOptionPane.ERROR_MESSAGE);
         }
         
         try
         {
            if (_qMgr != null)
               _qMgr.disconnect();
         }
         catch (MQException e)
         {
            JOptionPane.showMessageDialog( parent,
                                           "QMgrName = "+mqo.qMgrName+"\nQueue Name = "+mqo.qObject.qName+"\nCompletion Code = " + e.completionCode + "\nReason Code = " + e.reasonCode,
                                           "Error",
                                           JOptionPane.ERROR_MESSAGE);
         }
         
         /**
          * Hard exit now that the user knows what has happened.
          */
         if (majorFailure)
         {
            System.exit(99);
         }
      }
   }

   /**
    * The masterLoop() method will loop endlessly get status messages
    *
    * @return boolean
    */
   private void masterLoop(MQQueue inQ)
   {
      boolean innerLoop = true;
      startTime = timeIs();  // clock the original start time for a default value.
      MQMessage getMsg = null;
      int waitInterval = 100;

      MQGetMessageOptions getOptions = new MQGetMessageOptions();
      getOptions.options = MQC.MQGMO_WAIT + MQC.MQGMO_BROWSE_FIRST + MQC.MQGMO_CONVERT + MQC.MQGMO_ACCEPT_TRUNCATED_MSG + MQC.MQGMO_FAIL_IF_QUIESCING;
      getOptions.waitInterval = waitInterval;

      try
      {
         while (working)
         {
            cells = new ArrayList();
            while (innerLoop)
            {
               try
               {
                  getMsg = new MQMessage();
                  inQ.get(getMsg, getOptions, 256);
                  
                  cells.add(handleParseStatusMsg(getMsg));
               }
               catch (MQException e)
               {
                  if ( (e.completionCode == 1 && e.reasonCode == MQException.MQRC_FORMAT_ERROR)           ||
                        (e.completionCode == 1 && e.reasonCode == MQException.MQRC_TRUNCATED_MSG_ACCEPTED) ||
                        (e.completionCode == 1 && e.reasonCode == MQException.MQRC_DBCS_ERROR)             )
                   {
                      //  that's fine.
                     cells.add(handleParseStatusMsg(getMsg));
                   }
                   else if (e.completionCode == 2 && e.reasonCode == MQException.MQRC_NO_MSG_AVAILABLE)
                   {
                      // ok
                      innerLoop = false;
                   }
                   else
                   {
                      JOptionPane.showMessageDialog( parent,
                                                     "QMgrName = "+mqo.qMgrName+"\nQueue Name = "+mqo.qObject.qName+"\nCompletion Code = " + e.completionCode + "\nReason Code = " + e.reasonCode,
                                                     "Error",
                                                     JOptionPane.ERROR_MESSAGE);
                      innerLoop = false;
                   }
               }
               
               getOptions = new MQGetMessageOptions();
               getOptions.options = MQC.MQGMO_WAIT + MQC.MQGMO_BROWSE_NEXT + MQC.MQGMO_CONVERT + MQC.MQGMO_ACCEPT_TRUNCATED_MSG + MQC.MQGMO_FAIL_IF_QUIESCING;
               getOptions.waitInterval = waitInterval; 
            }
            
            parent.updateTable(cells);

            nowSleeping();
            innerLoop = true;
            waitInterval = 2000;
         }
      }
      catch (ArrayIndexOutOfBoundsException abe)
      {
         JOptionPane.showMessageDialog( parent,
        						                "ArrayIndexOutOfBoundsException: Error: " + abe.getLocalizedMessage(),
                                        "Error",
      						                JOptionPane.ERROR_MESSAGE);
      }
      catch (NumberFormatException nfe)
      {
         JOptionPane.showMessageDialog( parent,
       						                "NumberFormatException: Error: " + nfe.getLocalizedMessage(),
                                        "Error",
      						                JOptionPane.ERROR_MESSAGE);
      }
      catch (Exception e)
      {
         JOptionPane.showMessageDialog( parent,
                                        "Exception: Error: \n" + e.getLocalizedMessage(),
                                        "MQGetStatusMessages: Error",
                                        JOptionPane.ERROR_MESSAGE);
      }
   }

   /**
    * Parse the status message
    * @param msg
    * @return newCells
    */
   private Object[] handleParseStatusMsg(MQMessage msg)
   {
      Object[] newCells = new Object[CWConstants.statusQcolumns];
      for (int i=0; i < CWConstants.statusQcolumns; i++)
         newCells[i] = new String("");
      
      newCells[CWConstants.statusQcolumns - 1] = msg.correlationId;

      try
      {
         int msgLength = msg.getMessageLength();
         byte[] b = new byte[msgLength];
         msg.readFully(b);
         String msgText = new String(b);
         
         StringTokenizer st = new StringTokenizer(msgText,",\n");

         for (int i=0; i < (CWConstants.statusQcolumns - 1);i++)
         {
            if (st.hasMoreTokens())
            {
               newCells[i] = st.nextToken().trim();
            }
         }
      }
      catch (IOException e)
      {
         JOptionPane.showMessageDialog( parent,
                                        "IOException: Error: \n" + e.getLocalizedMessage(),
                                        "Error",
                                        JOptionPane.ERROR_MESSAGE);
      }
      
      return newCells;
   }

   /**
    * Get the current time.
    * @return long (timestamp)
    */
   private long timeIs()
   {
      Date currentDate = new java.util.Date();
      return currentDate.getTime();
   }

   /**
    * Have a little snooze.
    */
   private void nowSleeping()
   {
      long  waitTime;
      endTime = timeIs();  // clock the 'end' time

      waitTime = (mqo.refreshRate * 1000) - (endTime - startTime);
      if (waitTime <= 0 )
         waitTime = 20;  // make a default wait of 20 ms

      try
      {
         /*
          * Check for user input every 1/4 second (250ms)
          */
         while ( (waitTime > 0) && (working))
         {
            Thread.sleep(250); // time in milliseconds
            waitTime -= 250;
         }
      }
      catch (InterruptedException ie) {}

      startTime = timeIs();  // save the time because we are beginning the work again.
   }

   /**
    * Someone wants the MQ PCF collection to stop.
    */
   public void stopPlease()
   {
      working = false;
   }
}
