package com.capitalware.ufmsm.popups;

import java.awt.*;
import java.awt.event.*;
import java.util.StringTokenizer;

import javax.swing.*;
import javax.swing.table.*;

import com.capitalware.ufmsm.objects.CWConstants;

/**
 * This class will display the results  
 *
 * @author Roger Lacroix
 * @version 1.0
 * @license Apache 2 License
 */
public class ActionDetailsDialog extends JDialog implements ActionListener, ComponentListener
{
   private JTextField   date;
   private JTextField   time;
   private JTextField   exitCode;
   private JTextField   actionName;
   private JTextField   actionLabel;
   private JTextField   event;
   //
   private JButton okButton;

   /**
    * Create this dialog with the given parent and title.
    *
    * @param parent window from which this dialog is launched
    * @param title the title for the dialog box window
    */
   public ActionDetailsDialog(JDialog parent, String title)
   {
      super(parent, title, false);

      setTitle(title);

      if (parent != null)
      {
          setLocationRelativeTo(parent);
      }
      // super calls dialogInit, so we don't need to do it again.
   }

   /**
    * Set the data for this particular action.
    * @param data
    */
   public void setData(Object[] data)
   {
      Object[][] cells = null;
      String[] columnNames = {"Name", "Value"};
      
      this.date.setText(((String)data[0]).substring(0,10));
      this.time.setText(((String)data[0]).substring(11));
      this.exitCode.setText((String)data[1]);
      this.actionName.setText((String)data[2]);
      this.actionLabel.setText((String)data[3]);
      this.event.setText((String)data[4]);
      
      String parms = (String)data[5];
      
      if ( (parms == null) || ("".equals(parms.trim())) )
      {
         cells = new Object[0][columnNames.length];
      }
      else
      {
         parms = parms.trim();
         
         // do we have a trailing comma? if so, remove it
         if( parms.length() == parms.lastIndexOf(','))
            parms = parms.substring(0, parms.length()-1);
         
         StringTokenizer st = new StringTokenizer(parms,",");
   
         cells = new Object[st.countTokens()][columnNames.length];
         
         int i=0;
         while (st.hasMoreTokens())
         {
            StringTokenizer st2 = new StringTokenizer(st.nextToken(),"=");
            
            cells[i] = new Object[2];
            
            cells[i][0] = st2.nextToken().trim();
            if (st2.hasMoreTokens())
               cells[i][1] = st2.nextToken().trim();
            
            i++;
         }
      }
      
      // set up table model
      DefaultTableModel model = new DefaultTableModel(cells, columnNames);
      
      JTable table = new JTable(model);

      GridBagConstraints gbc = new GridBagConstraints();
      // table
      gbc.anchor = GridBagConstraints.CENTER;
      gbc.fill = GridBagConstraints.BOTH;
      gbc.weightx = 1;
      gbc.weighty = 1;
      gbc.insets = new Insets(10,5,0,5);  // Top, Left, Bottom, Right
      JScrollPane JSp1 = new JScrollPane(table);
      
      JSp1.setPreferredSize(new Dimension(400,200));

      add(JSp1, gbc, 0, 6, 5, 1);  // c , r, cw, rh
      
      TableColumnModel columnModel = table.getColumnModel();
      TableColumn column = columnModel.getColumn(0);
      column.setPreferredWidth(125);
      column.setMaxWidth(125);
   }

   /**
    * Called by constructors to initialize the dialog.
    */
   protected void dialogInit()
   {
      super.dialogInit();
      addComponentListener(this);

      setSize(CWConstants.ADD_MIN_WIDTH, CWConstants.ADD_MIN_HEIGHT);     // w * h
      
      getContentPane().setLayout(new GridBagLayout());
      GridBagConstraints gbc = new GridBagConstraints();

      gbc.anchor = GridBagConstraints.WEST;
      gbc.weightx = 0;
      gbc.weighty = 0;
      gbc.fill = GridBagConstraints.NONE;
      gbc.insets = new Insets(0,0,0,0);  // Top, Left, Bottom, Right

      // Date label
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      JLabel dateLabel = new JLabel("Date:");
      dateLabel.setForeground(Color.black);
      add(dateLabel, gbc, 0, 0, 3, 1);   // c , r, cw, rh

      // Date field
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      date = new JTextField("", 8);
      date.setEditable(false);
      add(date, gbc, 3, 0, 2, 1);   // c , r, cw, rh

      // Time label
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      JLabel timeLabel = new JLabel("Time:");
      timeLabel.setForeground(Color.black);
      add(timeLabel, gbc, 0, 1, 3, 1);   // c , r, cw, rh

      // Time field
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      time = new JTextField("", 8);
      time.setEditable(false);
      add(time, gbc, 3, 1, 2, 1);   // c , r, cw, rh

      // Status label
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      JLabel statusLabel = new JLabel("Exit Code:");
      statusLabel.setForeground(Color.black);
      add(statusLabel, gbc, 0, 2, 3, 1);   // c , r, cw, rh

      // Status field
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      exitCode = new JTextField("", 8);
      exitCode.setEditable(false);
      add(exitCode, gbc, 3, 2, 2, 1);   // c , r, cw, rh

      // Action Name label
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      JLabel actionNameLabel = new JLabel("Action Name:");
      actionNameLabel.setForeground(Color.black);
      add(actionNameLabel, gbc, 0, 3, 3, 1);   // c , r, cw, rh

      // Action Name field
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      actionName = new JTextField("", 12);
      actionName.setEditable(false);
      add(actionName, gbc, 3, 3, 2, 1);   // c , r, cw, rh

      // "Action Label" label
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      JLabel actionLabelLabel = new JLabel("Action Label:");
      actionLabelLabel.setForeground(Color.black);
      add(actionLabelLabel, gbc, 0, 4, 3, 1);   // c , r, cw, rh

      // "Action Label" field
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      actionLabel = new JTextField("", 12);
      actionLabel.setEditable(false);
      add(actionLabel, gbc, 3, 4, 2, 1);   // c , r, cw, rh

      // Event label
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      JLabel eventLabel = new JLabel("Event:");
      eventLabel.setForeground(Color.black);
      add(eventLabel, gbc, 0, 5, 3, 1);   // c , r, cw, rh

      // Event field
      gbc.anchor = GridBagConstraints.WEST;
      gbc.insets = new Insets(5,10,0,10);  // Top, Left, Bottom, Right
      event = new JTextField("", 20);
      event.setEditable(false);
      add(event, gbc, 3, 5, 2, 1);   // c , r, cw, rh

      // Create Pane #3

      JPanel pane3 = new JPanel(new GridBagLayout());
      gbc.anchor = GridBagConstraints.CENTER;

      // Ok Button

      gbc.insets = new Insets(0,0,0,0);  // Top, Left, Bottom, Right
      okButton = new JButton("OK");
      okButton.setToolTipText("OK");
      okButton.setPreferredSize(new Dimension(70,25));
//      okButton.setBorder(BorderFactory.createRaisedBevelBorder());
      okButton.addActionListener(this);
      addToPane(pane3, okButton, gbc, 0, 0, 1, 1);

      gbc.anchor = GridBagConstraints.EAST;
      gbc.weightx = 0;
      gbc.weighty = 0;
      gbc.insets = new Insets(15,0,10,10);  // Top, Left, Bottom, Right
      gbc.fill = GridBagConstraints.NONE;
      add(pane3, gbc, 2, 12, 5, 1);     // c , r, cw, rh

      // Setup to handle keyboard actions - Enter & Esc

      KeyListener keyListener = (new KeyAdapter()
      {
         public void keyTyped(KeyEvent e)
         {
           // System.out.println("e.getKeyChar()=" + e.getKeyChar());
            if (e.getKeyChar() == KeyEvent.VK_ESCAPE ||
                (e.getSource() == okButton  && e.getKeyChar() == KeyEvent.VK_ENTER))
            {
               dialogCleanUp();
            }
         }
      });

      addKeyListener(keyListener);

      okButton.addKeyListener(keyListener);
      date.addKeyListener(keyListener);
      time.addKeyListener(keyListener);
      exitCode.addKeyListener(keyListener);
      actionName.addKeyListener(keyListener);

      actionLabel.addKeyListener(keyListener);

      addWindowListener(new WindowAdapter()
      {
         public void windowClosing(WindowEvent we)
         {
            dialogCleanUp();
         }
      });

      getOwner().setEnabled(false);
   }
   
   /**
    * Add a component to main pane.
    * @param c
    * @param gbc
    * @param x
    * @param y
    * @param w
    * @param h
    */
   private void add(Component c, GridBagConstraints gbc2, int x, int y, int w, int h)
   {
      gbc2.gridx = x;
      gbc2.gridy = y;
      gbc2.gridwidth = w;
      gbc2.gridheight = h;
      getContentPane().add(c, gbc2);
   }

   /**
    * Add a component a to JPanel.
    * @param p
    * @param c
    * @param gbc
    * @param x
    * @param y
    * @param w
    * @param h
    */
   private void addToPane(JPanel p, Component c, GridBagConstraints gbc, int x, int y, int w, int h)
   {
      gbc.gridx = x;
      gbc.gridy = y;
      gbc.gridwidth = w;
      gbc.gridheight = h;
      p.add(c, gbc);
   }

   /**
    * Handle the generated Action Event.
    */
   public void actionPerformed(ActionEvent event)
   {
      Object source = event.getSource();

      if (source == okButton)
      {
         dialogCleanUp();
      }
   }

   /**
    * Time to enable our parent and throw ourselves away.
    */
   private void dialogCleanUp()
   {
      getOwner().setEnabled(true);
      this.hide();
   }

   /**
    *
    * @param e
    */
   public void componentResized(ComponentEvent e)
   {
      int width = getWidth();
      int height = getHeight();

      //we check if either the width
      //or the height are below minimum

      boolean resize = false;

      if (width < CWConstants.ADD_MIN_WIDTH)
      {
         resize = true;
         width = CWConstants.ADD_MIN_WIDTH;
      }

      if (height < CWConstants.ADD_MIN_HEIGHT)
      {
         resize = true;
         height = CWConstants.ADD_MIN_HEIGHT;
      }

      if (resize)
      {
         setSize(width, height);
      }
   }

   public void componentMoved(ComponentEvent e)
   {
   }

   public void componentShown(ComponentEvent e)
   {
   }

   public void componentHidden(ComponentEvent e)
   {
   }
}
